const path = require('path')
const eslint = require('eslint')
const { CLIEngine } = eslint
const cwd = process.cwd()
const directories = cwd.split(path.sep)
const folder = directories[directories.length - 1]
const configPath = path.join(cwd, '.complexity.js')

/* 获取复杂度配置文件 */
function fetchConfig () {
  try {
    return require(configPath)
  } catch (e) {
    throw new Error('未找到复杂度配置文件')
  }
}

/*
* 初始化Eslint CLI
* @param complexityMaxValue: number 复杂度最大限制
*/
function initCLI (complexityMaxValue) {
  return new CLIEngine({
    rules: {
      complexity: [
        'error',
        { max: complexityMaxValue }
      ]
    },
    useEslintrc: true
  })
}

/* 计算复杂度 返回JSON */
function lxComplexity() {
  try {
    const { complexityMaxValue, globs } = fetchConfig()
    const cli = initCLI(+complexityMaxValue)
    const { results } = cli.executeOnFiles(globs) || []
    const list = []
    results.forEach((item) => {
      const { messages, filePath: currentPath } = item
      const filePath = currentPath.replace(cwd, folder)
      messages.forEach((messagesItem) => {
        // 函数名，所在行、所在列，复杂度，文件路径，message
        const {
          ruleId,
          line,
          column,
          message,
          nodeType
        } = messagesItem
        const matchMethods = message.match(/'(.+)'/)
        const method = matchMethods ? matchMethods[1] : '无'
        const complexityMatches = message.match(/.+of\s(\d+)\./)
        const complexityCount = complexityMatches ? +complexityMatches[1] : 0
        const record = {
          ruleId,
          method,
          complexity: complexityCount,
          line,
          column,
          message,
          nodeType
        }
        if (['complexity'].includes(ruleId)) {
          list.push({ file: filePath, messages: record })
        }
      })
    })
    return list.sort((a, b) => b.messages.complexity - a.messages.complexity)
  } catch (e) {
    const { message } = e
    throw new Error(`复杂度分析出错：${message}`)
  }
}

module.exports = lxComplexity
